// openhomm - open source clone of Heroes of Might and Magic III
// Copyright (C) 2009-2010 openhomm developers team (see AUTHORS)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
#include "precompiled.hpp"
#include "hrBorder.hpp"
#include "hrCache.hpp"

int hrBorder::getWidth() const
{
    return rect.width();
}

void hrBorder::setWidth(int w)
{
    rect.setWidth(w);
    resizeBorder(rect.width(), rect.height());
}

int hrBorder::getHeight() const
{
    return rect.height();
}

void hrBorder::setHeight(int h)
{
    rect.setHeight(h);
    resizeBorder(rect.width(), rect.height());
}

QString hrBorder::getName() const
{
    return name;
}

void hrBorder::setName(QString name)
{
    this->name = name;

    hrCache &cache = hrCache::Get();
    QImage im("vfs:/" + name);
    topLeft = cache.loadItem(im.copy(0, 0, 8, 8));
    left = cache.loadItem(im.copy(0, 100, 8, 128));
    top = cache.loadItem(im.copy(100, 0, 128, 8));
    bottomLeft = cache.loadItem(im.copy(0, 553, 53, 47));
    bottom = cache.loadItem(im.copy(53, 553, 128, 47));
    topRight = cache.loadItem(im.copy(602, 0, 5, 5));
    right = cache.loadItem(im.copy(602, 5, 5, 128));
    bottomRight = cache.loadItem(im.copy(553, 553, 53, 47));

    rect.setRect(0, 0, 800, 600);
    resizeBorder(800, 600);
}

void hrBorder::resizeBorder(int w, int h)
{
    items.clear();

    addItem(topLeft, 0, 0);

    for (int y = topLeft.height(); y < h; y += left.height())
        addItem(left, 0, y);

    for (int x = topLeft.width(); x < w; x += top.width())
        addItem(top, x, 0);

    addItem(bottomLeft, 0, h - bottomLeft.height());

    for (int x = bottomLeft.width(); x < w; x += bottom.width())
        addItem(bottom, x, h - bottom.height());

    addItem(topRight, w - topRight.width(), 0);

    for (int y = topRight.height(); y < h; y += right.height())
        addItem(right, w - right.width(), y);

    addItem(bottomRight, w - bottomRight.width(), h - bottomRight.height());
}

void hrBorder::addItem(hrGraphicsItem item, int x, int y)
{
    item.setPoint(QPoint(x, y));
    items.append(item);
}
