// openhomm - open source clone of Heroes of Might and Magic III
// Copyright (C) 2009-2010 openhomm developers team (see AUTHORS)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
#include "precompiled.hpp"
#include "hrUI.hpp"
#include "hrPushButton.hpp"
#include "hrBorder.hpp"

Q_SCRIPT_DECLARE_QMETAOBJECT(hrPushButton, QObject*)
Q_SCRIPT_DECLARE_QMETAOBJECT(hrBorder, QObject*)

hrUI::hrUI(QObject *parent) : QObject(parent)
{
    QScriptValue button = engine.scriptValueFromQMetaObject<hrPushButton>();
    engine.globalObject().setProperty("Button", button);

    QScriptValue border = engine.scriptValueFromQMetaObject<hrBorder>();
    engine.globalObject().setProperty("Border", border);

    addGlobalObject(this, "UI");
}

void hrUI::addGlobalObject(QObject *obj, QString name)
{
    QScriptValue val = engine.newQObject(obj);
    engine.globalObject().setProperty(name, val);
}

void hrUI::addWidget(hrWidget *widget)
{
    widgets.append(widget);
}

void hrUI::resize(int w, int h)
{
    for (int i = 0; i < widgets.size(); i++)
    {
        hrBorder *border = dynamic_cast<hrBorder*>(widgets.at(i));
        if (border)
        {
            border->resizeBorder(w, h);
        }
    }
}

void hrUI::onMousePress(const QPoint &p)
{
    for (int i = 0; i < widgets.size(); i++)
    {
        hrWidget *widget = widgets.at(i);
        if (widget->getRect().contains(p))
        {
            hrPushButton *button  = dynamic_cast<hrPushButton*>(widget);
            if (button)
            {
                button->onMousePress();
            }
        }
    }
}

void hrUI::onMouseRelease(const QPoint &p)
{
    for (int i = 0; i < widgets.size(); i++)
    {
        hrWidget *widget = widgets.at(i);
        if (widget->getRect().contains(p))
        {
            hrPushButton *button  = dynamic_cast<hrPushButton*>(widget);
            if (button)
            {
                button->onMouseRelease();
            }
        }
    }

}

QListIterator<hrGraphicsItem> hrUI::getItems()
{
    collect();
    return QListIterator<hrGraphicsItem>(items);
}

void hrUI::collect()
{
    items.clear();
    for (int i = 0; i < widgets.size(); i++)
    {
        hrWidget *widget = widgets.at(i);
        QVectorIterator<hrGraphicsItem> it = widget->getItems();
        while (it.hasNext())
        {
            hrGraphicsItem item = it.next();
            items.append(item);
        }
    }
}

void hrUI::script(QString path)
{
    QFile file(path);
    file.open(QIODevice::ReadOnly);
    if (file.error())
    {
        qDebug() << file.errorString();
    }
    QTextStream stream(&file);
    QString contents = stream.readAll();
    file.close();

    widgets.clear();

    QScriptValue result = engine.evaluate(contents, path);

    QScriptValueIterator it(result);
    while (it.hasNext())
    {
        it.next();
        qDebug() << it.name() << ": " << it.value().toString();
    }
}
